/************************************************************************
*									*
*  Objects of this class contain an array of objects of class Student	*
*  and can be used to keep track of student records for a course.	*
*									*
*  The following constructor and public methods are included in this 	*
*  class:								*
*									*
*	CourseList ( int size )						*
*	   constructor:  creates an array of students of length size 	*
*	   and sets current_course_size to 0				*
*									*
*	public boolean found_student_number ( String student_number ) 	*
*	   returns true if this student number is already listed in one *
*	   of the Student objects in the course_list array;		*
*	   returns false otherwise					*
*									*
*	public boolean full () 						*
*	   returns true if the course is full; false otherwise		*
*									*
*	public void add_student ( String student_name, 			*
*				 String student_number ) 		*
*	   adds a new student to the end of the course_list array	*
*									*
*	public void delete_student (String student_number) 		*
*	   finds a student with student number equal to the argument	*
*	   student_number, deletes that student from the course and 	*
*          moves the last student down to fill the gap created by 	*
*          deleting a student.						*
*									*
*	public void add_assignment_mark ( String student_number,	*
*                                         int which_assignment,		*
*                                         int assignment_mark ) 	*
*	   finds a Student object in the course_list array with 	*
*          student number student_number and changes that student's 	*
*	   mark for assignment which_assignment to assignment_mark	*
*									*
*	public void add_project_mark ( String student_number,		*
*				       int project_mark ) 		*
*	   finds a Student object in the course_list array with student *
*	   number student_number and changes that student's project 	*
*	   mark to project_mark						*
*									*
*	public void add_term_test_mark ( String student_number,		*
*				         int term_test_mark ) 		*
*	   finds a Student object in the course_list array with student *
*	   number student_number and changes that student's term test 	*
*	   mark to term_test_mark					*
*									*
*	public void add_exam_mark ( String student_number, 		*
*				    int exam_mark ) 			*
*	   finds a Student object in the course_list array with student *
*	   number student_number and changes that student's exam mark   *
*	   to exam_mark							*
*									*
*	public void compute_final_grades () 				*
*	   computes the final grade for each Student object in the 	*
*	   course_list array						*
*									*
*	public void report_grades () 					*
*	   reports the grades for each Student object in the 		*
*	   course_list array						*
*									*
*	public void report_stats () 					*
*	   computes the course average rounded to one decimal place and	*
*	   the mark distribution and reports these statistics		*
*									*
*                                       Ken Jackson, 13 March 1998.     *
*                                                                       *
************************************************************************/



class CourseList {

      private Student[] course_list ;
      private int current_course_size ;

      private final static int VOID_STUDENT_NUMBER = -999 ;



      CourseList ( int size ) {

         course_list = new Student[size] ;
         current_course_size = 0 ;

      } // end CourseList constructor



      private int student_number_index ( String student_number ) {

	 // This method returns the index of the first Student object 
         // object in the course_list array with the student number 
         // student_number.  It returns VOID_STUDENT_NUMBER if there 
         // is no Student object in the course_list array with the 
         // student number student_number.
         
         for ( int i = 0; i < current_course_size; i++ )
             if ( student_number.equals(course_list[i].get_student_number()) )
                return i ;

         return VOID_STUDENT_NUMBER ;

      } // end student_number_index



      public boolean found_student_number ( String student_number ) {

	 // This method returns true if this student number is already 
	 // listed in one of the Student objects in the course_list array.
	 // It returns false otherwise.

         // Note that the method student_number_index returns
         // VOID_STUDENT_NUMBER if the student is not in the course
         // and an index in the range 0 to current_course_size - 1
         // if it is.

         return student_number_index(student_number) 
                != VOID_STUDENT_NUMBER ;

      } // end found_student_number



      public boolean full () {

	 // This method returns true if the course is full.
	 // It returns false otherwise.

         return current_course_size == course_list.length ;

      } // end full



      public void add_student ( String student_name, String student_number ) {
         
         // Add a new Student object to the end of the course_list array.

         course_list[current_course_size] 
                = new Student ( student_name, student_number );
         current_course_size++;

      } // end add_student



      public void delete_student ( String student_number ) {

         // Find a Student object in the course_list array with student 
         // number equal to the argument student_number, delete that 
	 // student from the course and moves the last student down 
	 // to fill the gap created by deleting a student.
	 //
         // If there is more than one student with this student number
         // in the course, this method deletes the first one found only.
         // If there is no student with this student number in the course, 
	 // this method does nothing.

	 int student_index = student_number_index(student_number) ;

         if ( student_index != VOID_STUDENT_NUMBER ) {

            course_list[student_index] = course_list[current_course_size-1] ;

            current_course_size--;

         } // end if

      } // end delete_student



      public void add_assignment_mark ( String student_number,
                                        int which_assignment,
                                        int assignment_mark ) {

         // Find a Student object in the course_list array with 
	 // student number student_number and change that student's mark 
	 // for assignment which_assignment to assignment_mark.
	 //
         // If there is more than one student with this student number
         // in the course, this method changes the mark for the first
         // one found only.  If there is no student with this student 
         // number in the course, this method does nothing.

	 int student_index = student_number_index(student_number) ;

         if ( student_index != VOID_STUDENT_NUMBER ) 
            course_list[student_index].add_assignment_mark
               ( which_assignment, assignment_mark ) ;

      } // end add_assignment_mark



      public void add_project_mark ( String student_number,
                                     int project_mark ) {

         // Find a Student object in the course_list array with 
	 // student number student_number and change that student's 
	 // project mark to project_mark.
	 //
         // If there is more than one student with this student number
         // in the course, this method changes the mark for the first
         // one found only.  If there is no student with this student 
         // number in the course, this method does nothing.

	 int student_index = student_number_index(student_number) ;

         if ( student_index != VOID_STUDENT_NUMBER ) 
            course_list[student_index].add_project_mark ( project_mark ) ;

      } // end add_project_mark



      public void add_term_test_mark ( String student_number,
                                       int term_test_mark ) {

         // Find a Student object in the course_list array with 
	 // student number student_number and change that student's 
	 // term test mark to term_test_mark.
	 //
         // If there is more than one student with this student number
         // in the course, this method changes the mark for the first
         // one found only.  If there is no student with this student 
         // number in the course, this method does nothing.

	 int student_index = student_number_index(student_number) ;

         if ( student_index != VOID_STUDENT_NUMBER ) 
            course_list[student_index].add_term_test_mark ( term_test_mark ) ;

      } // end add_term_test_mark



      public void add_exam_mark ( String student_number, int exam_mark ) {

         // Find a Student object in the course_list array with 
	 // student number student_number and change that student's 
	 // exam mark to exam_mark.
	 //
         // If there is more than one student with this student number
         // in the course, this method changes the mark for the first
         // one found only.  If there is no student with this student
         // number in the course, this method does nothing.

         int student_index = student_number_index(student_number) ;

         if ( student_index != VOID_STUDENT_NUMBER )
            course_list[student_index].add_exam_mark ( exam_mark ) ;

      } // end add_exam_mark



      public void compute_final_grades () {

	 // Compute the final grade for each Student object in the 
	 // course_list array.

         for (int i = 0; i < current_course_size; i++ )
             course_list[i].compute_final_grade() ;

      } // end compute_final_grades



      public void report_grades () {

	 // Report the grades for each Student object in the course_list array.

         if ( current_course_size == 0 )
            System.out.println 
              ( "\nThere are no students in the course." ) ;
         else if ( current_course_size == 1 )
            System.out.println 
              ( "\nGrade report for 1 student in the course." ) ;
         else
            System.out.println 
              ( "\nGrade report for " + current_course_size
              + " students in the course." ) ;

         for (int i = 0; i < current_course_size; i++ )
             course_list[i].report_grades() ;

      } // end report_grades



      public void report_stats () {

	 // Compute the course average rounded to one decimal place
         // and the mark distribution and report these statistics.

         if ( current_course_size == 0 )
            System.out.println 
              ( "\nThere are no students in the course." ) ;
         else {
            int sum = 0 ;
            final int [] range = { -1, 34, 49, 59, 69, 79, 100 } ;
            int [] number_in_range = { 0, 0, 0, 0, 0, 0 };
            for (int i = 0; i < current_course_size; i++) {
                int final_grade = course_list[i].get_final_grade() ;
                sum += final_grade ;
                boolean number_in_range_not_updated = true ;
                for (int j = 0; j < number_in_range.length
                                && number_in_range_not_updated; j++) {
                    if ( final_grade <= range[j+1] ) {
                       number_in_range[j]++ ;
                       number_in_range_not_updated = false ;
                    } // end if
                } //end for
            } //end for
            System.out.println 
              ( "\nStatistics for the course." 
              + "\n\tThe average is: " 
                    + Helper.oneDecPlaceString 
                        ( (double) sum / current_course_size ) 
              + "\n\tThe mark distribution is:"
              + "\n\t    mark range \t number of students" );
            for (int j = 0; j < number_in_range.length; j++)
                System.out.println
                   ( "\t   " + Pad.blanks(range[j]+1,3) 
                   + " to " + Pad.blanks(range[j+1],3)
                   + "\t" + Pad.blanks(number_in_range[j],10) ) ;
         } // end if

      } // end report_stats

} // end CourseList
