/************************************************************************
*									*
*  Objects of this class contain a student's name, number and grades.   *
*  Also included are the valid grade ranges.				*
*                                                                       *
*  The following constructor and public methods are included in this    *
*  class:                                                               *
*                							*
*	Student ( String student_name, String student_number ) 		*
*	   contstructor:  create a Student object with this 		*
*	   student_name and student_number				*
*                							*
*	public static int get_min_assignment_mark_index () 		*
*	   returns the min assignment mark index 			*
*                							*
*	public static int get_max_assignment_mark_index () 		*
*	   returns the max assignment mark index 			*
*                							*
*	public static int get_min_assignment_mark () 			*
*	  returns the minimum valid assignment mark 			*
*                							*
*	public static int get_max_assignment_mark () 			*
*	  returns the maximum valid assignment mark			*
*                							*
*	public static int get_min_project_mark () 			*
*	  returns the minimum valid project mark 			*
*                							*
*	public static int get_max_project_mark () 			*
*	  returns the maximum valid project mark 			*
*                							*
*	public static int get_min_term_test_mark () 			*
*	  returns the minimum valid term test mark 			*
*                							*
*	public static int get_max_term_test_mark () 			*
*	  returns the maximum valid term test mark 			*
*                							*
*	public static int get_min_exam_mark () 				*
*	  returns the minimum valid exam mark	 			*
*                							*
*	public static int get_max_exam_mark () 				*
*	  returns the maximum valid exam mark	 			*
*                							*
*	public static boolean invalid_student_number 			*
*		      ( String student_number ) 			*
*	   returns true if the argument student_number is an invalid 	*
*	   student number;  otherwise it returns false			*
*                							*
*	public String get_student_number () 				*
*	   returns the student_number 					*
*                							*
*	public int get_final_grade () 					*
*	   returns the final_grade 					*
*                							*
*	public void add_assignment_mark ( int which_assignment,		*
*					  int  assignment_mark )	*
*	   checks that the arguments which_assignment and 		*
*	   assignment_mark are within the apporpiate ranges;		*
*	   if so, it changes this student's mark for this assignment;	*
*	   if not, it does nothing					*
*                							*
*	public void add_project_mark ( int project_mark ) 		*
*	   checks that the argument project_mark is within the 		*
*	   apporpiate range; 						*
*	   if so, it changes this student's project mark;		*
*	   if not, it does nothing					*
*                							*
*	public void add_term_test_mark ( int term_test_mark ) 		*
*	   checks that the argument term_test_mark is within the 	*
*	   apporpiate range;						*
*	   if so, it changes this student's term test mark;		*
*	   if not, it does nothing					*
*                							*
*	public void add_exam_mark ( int exam_mark ) 			*
*	   checks that the argument exam_mark is within the apporpiate  *
*	   range;							*
*	   if so, it changes this student's exam mark;			*
*	   if not, it does nothing					*
*                							*
*	public void compute_final_grade () 				*
*	   computes the final grade for the student by adding up their 	*
*	   course marks, taking each missing mark to be 0		*
*                							*
*	public void report_grades () 					*
*	   prints the student's name, number and grades, printng NA 	*
*	   instead of a number for each missing grade			*
*                							*
*                                       Ken Jackson, 13 March 1998.     *
*                                                                       *
************************************************************************/


class Student {

    private String student_name, student_number;

    private final static int MARK_UNINITIALIZED = -999 ;

    private final static int MAX_ASSIGNMENT_MARK = 10, 
                             MAX_PROJECT_MARK = 15,
                             MAX_TERM_TEST_MARK = 10,  
                             MAX_EXAM_MARK = 45,
                             MIN_MARK = 0,
                             MIN_ASSIGNMENT_MARK_INDEX = 1,
                             MAX_ASSIGNMENT_MARK_INDEX = 3 ;
                        

    // We never use element [0] of the assignment_mark array.
    // Hoewever, we need to give it a value so that we can access
    // assignment_mark[1],...,assignment_mark[3].
    private int[] assignment_mark = { MARK_UNINITIALIZED,  
                                      MARK_UNINITIALIZED, 
                                      MARK_UNINITIALIZED, 
                                      MARK_UNINITIALIZED } ;

    private int project_mark    =  MARK_UNINITIALIZED, 
                term_test_mark  =  MARK_UNINITIALIZED,
                exam_mark       =  MARK_UNINITIALIZED,
                final_grade     =  MARK_UNINITIALIZED;



    Student ( String student_name, String student_number ) {

       this.student_name = student_name ;
       this.student_number = student_number ;

       // Note that the grades are all set to MARK_UNINITIALIZED above.

    } //end Student constructor
       


    public static int get_min_assignment_mark_index () {
       return MIN_ASSIGNMENT_MARK_INDEX ;
    }



    public static int get_max_assignment_mark_index () {
       return MAX_ASSIGNMENT_MARK_INDEX ;
    }



    public static int get_min_assignment_mark () {
       return MIN_MARK ;
    }



    public static int get_max_assignment_mark () {
       return MAX_ASSIGNMENT_MARK ;
    }

    public static int get_min_project_mark () {
       return MIN_MARK ;
    }



    public static int get_max_project_mark () {
       return MAX_PROJECT_MARK ;
    }



    public static int get_min_term_test_mark () {
       return MIN_MARK ;
    }



    public static int get_max_term_test_mark () {
       return MAX_TERM_TEST_MARK ;
    }


    public static int get_min_exam_mark () {
       return MIN_MARK ;
    }



    public static int get_max_exam_mark () {
       return MAX_EXAM_MARK ;
    }
 




    public static boolean invalid_student_number (String student_number) {

      // This method returns true if the argument student_number is 
      // an invalid student number.  Otherwise it returns false.
      //
      // It considers a string to be an invalid student number
      // if its length is not 9 or if one of the characters in the
      // string is not a digit.  All other strings are considered
      // to be valid.

      final int VALID_STUDENT_NUMBER_LENGTH = 9 ;

      if ( student_number.length() != VALID_STUDENT_NUMBER_LENGTH )
         return true ;
      else {
         for ( int i = 0; i < student_number.length(); i++ )
             if ( student_number.charAt(i) < '0' 
               || student_number.charAt(i) > '9' )
                return true ;

         return false ;
      }

   } // end invalid_student_number


    public String get_student_number () {
       return student_number ;
    }




    public int get_final_grade () {
       return final_grade == MARK_UNINITIALIZED ? 0 : final_grade ;
    }
 



   public void add_assignment_mark ( int which_assignment, 
                                     int assignment_mark ) {

      // This routine checks that the arguments which_assignment
      // and assignment_mark are within the apporpiate ranges.
      // If they are, it changes this student's mark for this assignment.
      // If they are not, it does nothing.

      if ( MIN_ASSIGNMENT_MARK_INDEX <= which_assignment
        && which_assignment <= MAX_ASSIGNMENT_MARK_INDEX 
        && MIN_MARK <= assignment_mark
        && assignment_mark <= MAX_ASSIGNMENT_MARK )
           
           this.assignment_mark[which_assignment] = assignment_mark ;

   } // end add_assignment_mark


   public void add_project_mark ( int project_mark ) {

      // This routine checks that the argument project_mark
      // is within the apporpiate range.
      // If it is, it changes this student's project mark.
      // If it is not, it does nothing.

      if ( MIN_MARK <= project_mark
        && project_mark <= MAX_PROJECT_MARK )
          
           this.project_mark = project_mark ;

   } // end add_project_mark



   public void add_term_test_mark ( int term_test_mark ) {

      // This routine checks that the argument term_test_mark
      // is within the apporpiate range.
      // If it is, it changes this student's term test mark.
      // If it is not, it does nothing.

      if ( MIN_MARK <= term_test_mark
        && term_test_mark <= MAX_TERM_TEST_MARK )
         
           this.term_test_mark = term_test_mark ;

   } // end add_term_test_mark



   public void add_exam_mark ( int exam_mark ) {

      // This routine checks that the argument exam_mark
      // is within the apporpiate range.
      // If it is, it changes this student's exam mark.
      // If it is not, it does nothing.

      if ( MIN_MARK <= exam_mark
        && exam_mark <= MAX_EXAM_MARK )

           this.exam_mark = exam_mark ;

   } // end add_exam_mark



   public void compute_final_grade () {

      // This routine computes the final grade for the student
      // by adding up their course marks, taking each missing mark
      // to be 0.
      
      final_grade = 0 ;
   
      for (int i  = MIN_ASSIGNMENT_MARK_INDEX;
               i <= MAX_ASSIGNMENT_MARK_INDEX;
               i++ )
          final_grade += assignment_mark[i] == MARK_UNINITIALIZED 
                         ? 0 : assignment_mark[i] ;
                
       final_grade += project_mark == MARK_UNINITIALIZED 
                      ? 0 : project_mark ;
                            
       final_grade += term_test_mark == MARK_UNINITIALIZED 
                      ? 0 : term_test_mark ;
                            
       final_grade += exam_mark == MARK_UNINITIALIZED 
                      ? 0 : exam_mark ;
                            
   } // end compute_final_grade



   public void report_grades () {

      // This routine prints the student's name, number and grades,
      // printng NA instead of a number for each missing grade.
      //
      // Note that this routine does not recompute the final grade,
      // so, if the marks have been changed since the final grade
      // was last computed, the final grade will be wrong.
      
      System.out.println
         ( "\n   " + student_name 
         + " (# " + student_number + ")" );
   
      for (int i  = MIN_ASSIGNMENT_MARK_INDEX;
               i <= MAX_ASSIGNMENT_MARK_INDEX; i++ ) {

          System.out.print ( "\tassignment mark " + i + ":\t" ) ;
          if ( assignment_mark[i] == MARK_UNINITIALIZED )
             System.out.println ( "NA" ) ;
          else
             System.out.println ( assignment_mark[i] ) ;
       } // end for
                
       System.out.print ( "\tproject mark:\t\t" ) ;
       if ( project_mark == MARK_UNINITIALIZED )
          System.out.println ( "NA" ) ;
       else
          System.out.println ( project_mark ) ;

       System.out.print ( "\tterm test mark:\t\t" ) ;
       if ( term_test_mark == MARK_UNINITIALIZED )
          System.out.println ( "NA" ) ;
       else
          System.out.println ( term_test_mark ) ;

       System.out.print ( "\texam mark:\t\t" ) ;
       if ( exam_mark == MARK_UNINITIALIZED )
          System.out.println ( "NA" ) ;
       else
          System.out.println ( exam_mark ) ;

       System.out.print ( "\tfinal grade:\t\t" ) ;
       if ( final_grade == MARK_UNINITIALIZED )
          System.out.println ( "NA" ) ;
       else
          System.out.println ( final_grade ) ;

   } // end report_grades

} // end Student class
